/**
 * // Copyright (C) 2003-2009 by Object Mentor, Inc. All rights reserved.
 * // Released under the terms of the CPL Common Public License version 1.0.
 */
package fr.opensagres.fitnesse.widgets;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.sonatype.aether.artifact.Artifact;
import org.sonatype.aether.util.StringUtils;
import org.sonatype.aether.util.artifact.DefaultArtifact;

import fitnesse.html.HtmlUtil;
import fitnesse.wiki.PageData;
import fitnesse.wikitext.WidgetBuilder;
import fitnesse.wikitext.widgets.ClasspathWidget;
import fitnesse.wikitext.widgets.ParentWidget;
import fr.opensagres.fitnesse.widgets.internal.Aether;
import fr.opensagres.fitnesse.widgets.internal.SettingsBasedAether;

/**
 * @author pascalleclercq 
 * {@link http://github.com/pascalleclercq/aether-fitnesse-widget} for instructions
 * 
 * @see ClasspathWidget
 * 
 */
public class ArtifactWidget extends ClasspathWidget {

	static {
		PageData.classpathWidgetBuilder = new WidgetBuilder(new Class[] { ArtifactWidget.class, ClasspathWidget.class });
	}

	public static final String REGEXP = "^!artifact [^\r\n]*";
	private static final Pattern pattern = Pattern.compile("^!artifact (.*)");

	private String coords;

	/**
	 * @param parent
	 * @param inputText
	 *            {@code !artifact <groupId>:<artifactId>[:<extension>[:<classifier>]]:<version>}
	 * @throws Exception
	 */
	public ArtifactWidget(ParentWidget parent, String inputText) throws Exception {
		super(parent, "");

		Matcher matcher = pattern.matcher(inputText);
		coords = findCoordinate(matcher);

	}

	private static final String USER_HOME = System.getProperty("user.home");

	private static final File USER_MAVEN_CONFIGURATION_HOME = new File(USER_HOME, ".m2");

	private static final File DEFAULT_USER_LOCAL_REPOSITORY = new File(USER_MAVEN_CONFIGURATION_HOME, "repository");

	@Override
	public String render() throws Exception {
		try {
			return HtmlUtil.metaText("classpath: " + getText());
		} catch (Exception e) {
			// e.printStackTrace();
			return e.getMessage() + "\n" + e.getCause() + " please check check that your artifiact is installed or is available in your LOCAL_REPO parameter";
		}

	}

	@Override
	public String getText() throws Exception {
		 Aether aether =null;
		if(isUseSettingsXml()){
			aether = new SettingsBasedAether();
			
		} else {
			aether = new Aether();
			aether.setLocalRepository(getLocalRepo());
			aether.setRemoteRepositories(getRemoteRepo());
		}
		  
		final Artifact artifact = new DefaultArtifact(coords);

		
		return aether.resolve(artifact);

	}

	private boolean isUseSettingsXml() throws Exception{
		
		String useSettingsXmlStr = getWikiPage().getData().getVariable("USE_SETTINGS_XML");
		
		
		return Boolean.parseBoolean(useSettingsXmlStr);
	}
	private List<String> getRemoteRepo() throws Exception {
		List<String> result = new ArrayList<String>();

		String remoteReposStr = getWikiPage().getData().getVariable("REMOTE_REPO");
		if (!StringUtils.isEmpty(remoteReposStr)) {
			String[] array = remoteReposStr.split(";");
			for (String repo : array) {
				result.add(repo.trim());
			}

		}
		
		return result;

	}

	public String getLocalRepo() throws Exception {
		// this is the only way to be sure variable will be initialized during
		// "test"...
		String localRepo = getWikiPage().getData().getVariable("LOCAL_REPO");
		if (localRepo == null) {
			localRepo = DEFAULT_USER_LOCAL_REPOSITORY.getAbsolutePath();
		}

		return localRepo;
	}

	public String asWikiText() throws Exception {
		return "!artifact " + coords;
	}

	private String findCoordinate(Matcher matcher) {
		if (matcher.find())
			return matcher.group(1);
		else
			return "";
	}

}
